@@ -2,11 +2,14 @@
 
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
+from __future__ import annotations
+
 import json
 import os
 from glob import glob
+from typing import Any
 
-import json5  # type:ignore
+import json5  # type:ignore[import-untyped]
 from jsonschema import Draft4Validator as Validator
 from jsonschema import ValidationError
 from jupyter_server import _tz as tz
@@ -20,7 +23,12 @@ from .translation_utils import DEFAULT_LOCALE, L10N_SCHEMA_NAME, SYS_LOCALE, is_
 SETTINGS_EXTENSION = ".jupyterlab-settings"
 
 
-def _get_schema(schemas_dir, schema_name, overrides, labextensions_path):
+def _get_schema(
+    schemas_dir: str,
+    schema_name: str,
+    overrides: dict[str, Any],
+    labextensions_path: list[str] | None,
+) -> tuple[dict[str, Any], str]:
     """Returns a dict containing a parsed and validated JSON schema."""
     notfound_error = "Schema not found: %s"
     parse_error = "Failed parsing schema (%s): %s"
@@ -68,7 +76,7 @@ def _get_schema(schemas_dir, schema_name, overrides, labextensions_path):
     return schema, version
 
 
-def _get_user_settings(settings_dir, schema_name, schema):
+def _get_user_settings(settings_dir: str, schema_name: str, schema: Any) -> dict[str, Any]:
     """
     Returns a dictionary containing the raw user settings, the parsed user
     settings, a validation warning for a schema, and file times.
@@ -84,8 +92,8 @@ def _get_user_settings(settings_dir, schema_name, schema):
 
     if os.path.exists(path):
         stat = os.stat(path)
-        last_modified = tz.utcfromtimestamp(stat.st_mtime).isoformat()
-        created = tz.utcfromtimestamp(stat.st_ctime).isoformat()
+        last_modified = tz.utcfromtimestamp(stat.st_mtime).isoformat()  # type:ignore[no-untyped-call]
+        created = tz.utcfromtimestamp(stat.st_ctime).isoformat()  # type:ignore[no-untyped-call]
         with open(path, encoding="utf-8") as fid:
             try:  # to load and parse the settings file.
                 raw = fid.read() or raw
@@ -108,7 +116,7 @@ def _get_user_settings(settings_dir, schema_name, schema):
     )
 
 
-def _get_version(schemas_dir, schema_name):
+def _get_version(schemas_dir: str, schema_name: str) -> str:
     """Returns the package version for a given schema or 'N/A' if not found."""
 
     path = _path(schemas_dir, schema_name)
@@ -123,23 +131,25 @@ def _get_version(schemas_dir, schema_name):
 
 
 def _list_settings(
-    schemas_dir,
-    settings_dir,
-    overrides,
-    extension=".json",
-    labextensions_path=None,
-    translator=None,
-):
+    schemas_dir: str,
+    settings_dir: str,
+    overrides: dict[str, Any],
+    extension: str = ".json",
+    labextensions_path: list[str] | None = None,
+    translator: Any = None,
+    ids_only: bool = False,
+) -> tuple[list[Any], list[Any]]:
     """
     Returns a tuple containing:
      - the list of plugins, schemas, and their settings,
-       respecting any defaults that may have been overridden.
+       respecting any defaults that may have been overridden if `ids_only=False`,
+       otherwise a list of dict containing only the ids of plugins.
      - the list of warnings that were generated when
        validating the user overrides against the schemas.
     """
 
-    settings = {}
-    federated_settings = {}
+    settings: dict[str, Any] = {}
+    federated_settings: dict[str, Any] = {}
     warnings = []
 
     if not os.path.exists(schemas_dir):
@@ -156,19 +166,21 @@ def _list_settings(
         rel_schema_dir, schema_base = os.path.split(rel_path)
         _id = schema_name = ":".join(
             [rel_schema_dir, schema_base[: -len(extension)]]  # Remove file extension.
-        ).replace(
-            "\\", "/"
-        )  # Normalize slashes.
-        schema, version = _get_schema(schemas_dir, schema_name, overrides, None)
-        if translator is not None:
-            schema = translator(schema)
-        user_settings = _get_user_settings(settings_dir, schema_name, schema)
+        ).replace("\\", "/")  # Normalize slashes.
 
-        if user_settings["warning"]:
-            warnings.append(user_settings.pop("warning"))
+        if ids_only:
+            settings[_id] = dict(id=_id)
+        else:
+            schema, version = _get_schema(schemas_dir, schema_name, overrides, None)
+            if translator is not None:
+                schema = translator(schema)
+            user_settings = _get_user_settings(settings_dir, schema_name, schema)
 
-        # Add the plugin to the list of settings.
-        settings[_id] = dict(id=_id, schema=schema, version=version, **user_settings)
+            if user_settings["warning"]:
+                warnings.append(user_settings.pop("warning"))
+
+            # Add the plugin to the list of settings.
+            settings[_id] = dict(id=_id, schema=schema, version=version, **user_settings)
 
     if labextensions_path is not None:
         schema_paths = []
@@ -187,24 +199,27 @@ def _list_settings(
             rel_schema_dir, schema_base = os.path.split(rel_path)
             _id = schema_name = ":".join(
                 [rel_schema_dir, schema_base[: -len(extension)]]  # Remove file extension.
-            ).replace(
-                "\\", "/"
-            )  # Normalize slashes.
+            ).replace("\\", "/")  # Normalize slashes.
 
             # bail if we've already handled the highest federated setting
             if _id in federated_settings:
                 continue
 
-            schema, version = _get_schema(
-                schemas_dir, schema_name, overrides, labextensions_path=labextensions_path
-            )
-            user_settings = _get_user_settings(settings_dir, schema_name, schema)
+            if ids_only:
+                federated_settings[_id] = dict(id=_id)
+            else:
+                schema, version = _get_schema(
+                    schemas_dir, schema_name, overrides, labextensions_path=labextensions_path
+                )
+                user_settings = _get_user_settings(settings_dir, schema_name, schema)
 
-            if user_settings["warning"]:
-                warnings.append(user_settings.pop("warning"))
+                if user_settings["warning"]:
+                    warnings.append(user_settings.pop("warning"))
 
-            # Add the plugin to the list of settings.
-            federated_settings[_id] = dict(id=_id, schema=schema, version=version, **user_settings)
+                # Add the plugin to the list of settings.
+                federated_settings[_id] = dict(
+                    id=_id, schema=schema, version=version, **user_settings
+                )
 
     settings.update(federated_settings)
     settings_list = [settings[key] for key in sorted(settings.keys(), reverse=True)]
@@ -212,7 +227,9 @@ def _list_settings(
     return (settings_list, warnings)
 
 
-def _override(schema_name, schema, overrides):
+def _override(
+    schema_name: str, schema: dict[str, Any], overrides: dict[str, Any]
+) -> dict[str, Any]:
     """Override default values in the schema if necessary."""
     if schema_name in overrides:
         defaults = overrides[schema_name]
@@ -232,7 +249,9 @@ def _override(schema_name, schema, overrides):
     return schema
 
 
-def _path(root_dir, schema_name, make_dirs=False, extension=".json"):
+def _path(
+    root_dir: str, schema_name: str, make_dirs: bool = False, extension: str = ".json"
+) -> str:
     """
     Returns the local file system path for a schema name in the given root
     directory. This function can be used to filed user overrides in addition to
@@ -259,14 +278,15 @@ def _path(root_dir, schema_name, make_dirs=False, extension=".json"):
     return path
 
 
-def _get_overrides(app_settings_dir):
+def _get_overrides(app_settings_dir: str) -> tuple[dict[str, Any], str]:
     """Get overrides settings from `app_settings_dir`.
 
     The ordering of paths is:
     - {app_settings_dir}/overrides.d/*.{json,json5} (many, namespaced by package)
     - {app_settings_dir}/overrides.{json,json5} (singleton, owned by the user)
     """
-    overrides: dict
+    overrides: dict[str, Any]
+    error: str
     overrides, error = {}, ""
 
     overrides_d = os.path.join(app_settings_dir, "overrides.d")
@@ -297,27 +317,28 @@ def _get_overrides(app_settings_dir):
                 for plugin_id, config in path_overrides.items():
                     recursive_update(overrides.setdefault(plugin_id, {}), config)
             except Exception as e:
-                error = e  # type:ignore
+                error = e  # type:ignore[assignment]
 
     # Allow `default_settings_overrides.json` files in <jupyter_config>/labconfig dirs
     # to allow layering of defaults
     cm = ConfigManager(config_dir_name="labconfig")
 
-    for plugin_id, config in cm.get("default_setting_overrides").items():
+    for plugin_id, config in cm.get("default_setting_overrides").items():  # type:ignore[no-untyped-call]
         recursive_update(overrides.setdefault(plugin_id, {}), config)
 
     return overrides, error
 
 
 def get_settings(
-    app_settings_dir,
-    schemas_dir,
-    settings_dir,
-    schema_name="",
-    overrides=None,
-    labextensions_path=None,
-    translator=None,
-):
+    app_settings_dir: str,
+    schemas_dir: str,
+    settings_dir: str,
+    schema_name: str = "",
+    overrides: dict[str, Any] | None = None,
+    labextensions_path: list[str] | None = None,
+    translator: Any = None,
+    ids_only: bool = False,
+) -> tuple[dict[str, Any], list[Any]]:
     """
     Get settings.
 
@@ -343,9 +364,10 @@ def get_settings(
     -------
     tuple
         The first item is a dictionary with a list of setting if no `schema_name`
-        was provided, otherwise it is a dictionary with id, raw, scheme, settings
-        and version keys. The second item is a list of warnings. Warnings will
-        either be a list of i) strings with the warning messages or ii) `None`.
+        was provided (only the ids if `ids_only=True`), otherwise it is a dictionary
+        with id, raw, scheme, settings and version keys.
+        The second item is a list of warnings. Warnings will either be a list of
+        i) strings with the warning messages or ii) `None`.
     """
     result = {}
     warnings = []
@@ -367,6 +389,7 @@ def get_settings(
             overrides,
             labextensions_path=labextensions_path,
             translator=translator,
+            ids_only=ids_only,
         )
         result = {
             "settings": settings_list,
@@ -376,13 +399,13 @@ def get_settings(
 
 
 def save_settings(
-    schemas_dir,
-    settings_dir,
-    schema_name,
-    raw_settings,
-    overrides,
-    labextensions_path=None,
-):
+    schemas_dir: str,
+    settings_dir: str,
+    schema_name: str,
+    raw_settings: str,
+    overrides: dict[str, Any],
+    labextensions_path: list[str] | None = None,
+) -> None:
     """
     Save ``raw_settings`` settings for ``schema_name``.
 
@@ -419,7 +442,14 @@ def save_settings(
 class SchemaHandler(APIHandler):
     """Base handler for handler requiring access to settings."""
 
-    def initialize(self, app_settings_dir, schemas_dir, settings_dir, labextensions_path, **kwargs):
+    def initialize(
+        self,
+        app_settings_dir: str,
+        schemas_dir: str,
+        settings_dir: str,
+        labextensions_path: list[str] | None,
+        **kwargs: Any,
+    ) -> None:
         """Initialize the handler."""
         super().initialize(**kwargs)
         self.overrides, error = _get_overrides(app_settings_dir)
@@ -432,7 +462,7 @@ class SchemaHandler(APIHandler):
             overrides_warning = "Failed loading overrides: %s"
             self.log.warning(overrides_warning % str(error))
 
-    def get_current_locale(self):
+    def get_current_locale(self) -> str:
         """
         Get the current locale as specified in the translation-extension settings.
 
